import pytest

from minos.common.configuration.config import MinosConfig
from minos.common.exceptions import MinosConfigException


def test_config_ini_fail():
    with pytest.raises(MinosConfigException):
        instance = MinosConfig(path='./test_fail_config.ini')


def test_config_kafka():
    provider_instance = MinosConfig(path='./tests/test_config.ini')
    kafka = provider_instance.kafka
    assert kafka.host == "kafka.minos.run"
    assert kafka.port == 9092


def test_config_rest():
    provider_instance = MinosConfig(path='./tests/test_config.ini')
    rest = provider_instance.rest
    assert rest.host == "localhost"
    assert rest.port == 8900


def test_config_microservice():
    provider_instance = MinosConfig(path='./tests/test_config.ini')
    microservice = provider_instance.microservice
    assert isinstance(microservice.events, list) == True
    assert isinstance(microservice.saga, list) == True
    assert microservice.events[0] == "OrderAdded"
    assert microservice.saga[0] == "CreateOrder"



import collections
import configparser
import os
import typing as t
from minos.common.exceptions import MinosConfigException
from minos.common.logs import log

KAFKA = collections.namedtuple("Kafka", "host port")
REST = collections.namedtuple("Rest", "host port")
MICROSERVICE = collections.namedtuple("Microservice", "name events saga commands")


class MinosConfig(object):
    __slots__ = "_parser"

    def __init__(self, path: str):
        if os.path.isfile(path):
            self._parser = configparser.ConfigParser()
            self._parser.read(path)
        else:
            raise MinosConfigException(f"file {path} does not exit")

    @property
    def kafka(self):
        return KAFKA(
            host=self._get("host", "kafka"),
            port=self._get("port", "kafka")
        )

    @property
    def rest(self):
        return REST(
            host=self._get("host", "rest"),
            port=self._get("port", "rest")
        )

    @property
    def microservice(self):
        return MICROSERVICE(
            name=self._get("name", "microservice"),
            events=self._get("events", "microservice"),
            saga=self._get("saga", "microservice"),
            commands=self._get("commands", "microservice")
        )

    def _get(self, key: str, section: str, default: t.Any = None) -> t.Union[None, t.List, str]:

        if self._parser.has_section(section) and self._parser.has_option(section, key):
            key_val = self._parser[section][key]
            # check if key val is an array of values
            if "," in key_val:
                log.debug(f"{key_val} is an array of values")
                return_val = key_val.split(",")
            else:
                log.debug(f"{key_val} is a string")
                # check if is a numeric value
                if key_val.isdecimal():
                    return_val = int(key_val)
                else:
                    return_val = key_val
            return return_val
        else:
            return default



import abc


class MinosStorage(abc.ABC):

    @abc.abstractmethod
    def add(self, **kwargs):
        raise NotImplementedError

    @abc.abstractmethod
    def get(self, **kwargs):
        raise NotImplementedError

    @abc.abstractmethod
    def delete(self, **kwargs):
        raise NotImplementedError

    @abc.abstractmethod
    def update(self, **kwargs):
        raise NotImplementedError

    @abc.abstractmethod
    def build(self, **kwargs):
        raise NotImplementedError



import pytest

from minos.common.exceptions import MinosImportException
from minos.common.importlib import import_module


def test_import_module():
    object_class = import_module("tests.ImportedModule.ImportedClassTest")
    instance_class = object_class()
    assert instance_class.return_test_example() == "test passed"


def test_import_module_exception():
    with pytest.raises(MinosImportException):
        object_class = import_module("tests.ImportedModuleFail.ImportedClassTest")
        assert True == True




import io
import typing as t
import avro
import orjson
from avro.datafile import DataFileReader, DataFileWriter
from avro.io import DatumReader, DatumWriter

from minos.common.exceptions import MinosProtocolException
from minos.common.logs import log
from minos.common.protocol.abstract import MinosBinaryProtocol


class MinosAvroProtocol(MinosBinaryProtocol):


    @classmethod
    def encode(cls, headers: t.Dict, body: t.Any = None) -> bytes:
        """
        encoder in avro
        all the headers are converted in fields with doble underscore name
        the body is a set fields coming from the data type.
        """

        #prepare the headers
        schema = {
            "type": "record",
            "namespace": 'org.minos.protocol',
            "name": "message",
            "fields": [
                {
                    "name": "headers",
                    "type": {
                        "type": "map",
                        "values": ["string", "int", "bytes", "long", "float", "boolean"]
                    }
                },
                {
                    "name": "body",
                    "type": [
                        "null", "string", "int", "bytes", "boolean", "float", "long",
                        {
                            "type": "array",
                            "items": ["int", "string", "bytes", "long", "boolean", "float"]
                        },
                         {
                             "type": "map",
                             "values": [
                                 "null", "int", "string", "bytes", "boolean", "float", "long",
                                 {"type": "array", "items": [
                                     "string", "int", "bytes", "long", "boolean", "float",
                                     {"type": "map", "values": ["string", "int", "bytes", "long", "boolean", "float"]}
                                 ]},
                                 {"type": "map", "values": ["string", "int", "bytes", "long", "boolean", "float"]}
                             ]
                         }
                    ]
                }
            ]
        }
        schema_json = orjson.dumps(schema)
        schema_bytes = avro.schema.parse(schema_json)
        final_data = {}
        final_data['headers'] = headers
        if body:
            final_data['body'] = body
        with io.BytesIO() as f:
            writer = DataFileWriter(f, DatumWriter(), schema_bytes)
            writer.append(final_data)
            writer.flush()
            f.seek(0)
            content_bytes = f.getvalue()
            writer.close()
            return content_bytes


    @classmethod
    def decode(cls, data: bytes) -> t.Dict:
        data_return: t.Dict = {}
        try:
            message_avro = DataFileReader(io.BytesIO(data), DatumReader())
            data_return['headers'] = {}
            for schema_dict in message_avro:
                log.debug(f"Avro: get the request/response in dict format")
                data_return['headers'] = schema_dict['headers']
                # check wich type is body
                if "body" in schema_dict:
                    if isinstance(schema_dict['body'], dict):
                        data_return['body'] = {}
                    elif isinstance(schema_dict['body'], list):
                        data_return['body'] = []
                    data_return['body'] = schema_dict['body']
            return data_return
        except Exception:
            raise MinosProtocolException("Error decoding string, check if is a correct Avro Binary data")


class MinosAvroValuesDatabase(MinosBinaryProtocol):

    @classmethod
    def encode(cls, value: t.Any, schema: t.Dict = None) -> bytes:
        """
        encoder in avro for database Values
        all the headers are converted in fields with doble underscore name
        the body is a set fields coming from the data type.
        """

        # prepare the headers
        if not schema:
            schema_val = {
                "type": "record",
                "namespace": 'org.minos.protocol.database',
                "name": "value",
                "fields": [
                    {
                        "name": "content",
                        "type": [
                            "null", "string", "int", "bytes", "boolean", "float", "long",
                            {
                                "type": "array",
                                "items": ["int", "string", "bytes", "long", "boolean", "float"]
                            },
                             {
                                 "type": "map",
                                 "values": [
                                     "null", "int", "string", "bytes", "boolean", "float", "long",
                                     {"type": "array", "items": [
                                         "string", "int", "bytes", "long", "boolean", "float",
                                         {"type": "map", "values": ["string", "int", "bytes", "long", "boolean", "float"]}
                                     ]},
                                     {"type": "map", "values": ["string", "int", "bytes", "long", "boolean", "float"]}
                                 ]
                             }
                        ]
                    }
                ]
            }
        else:
            schema_val = schema

        schema_json = orjson.dumps(schema_val)
        schema_bytes = avro.schema.parse(schema_json)
        final_data = {}
        final_data['content'] = value
        with io.BytesIO() as f:
            writer = DataFileWriter(f, DatumWriter(), schema_bytes)
            writer.append(final_data)
            writer.flush()
            f.seek(0)
            content_bytes = f.getvalue()
            writer.close()
            return content_bytes

    @classmethod
    def decode(cls, data: bytes) -> t.Dict:
        try:
            message_avro = DataFileReader(io.BytesIO(data), DatumReader())
            for schema_dict in message_avro:
                log.debug(f"Avro Database: get the values data")
                data_return = schema_dict['content']
            return data_return
        except Exception:
            raise MinosProtocolException("Error decoding string, check if is a correct Avro Binary data")



import importlib
import typing as t

import six

from minos.common.exceptions import MinosImportException


def import_module(module: str) -> t.Callable:
    """Import the given module from a package"""
    try:
        if '.' in module:
            parts = module.split('.')
            name = '.'.join(parts[:-1])

        module_ref = importlib.import_module(name)
        kallable = getattr(module_ref, parts[-1])
        if not six.callable(kallable):
            raise TypeError("The module is not callable")
        return kallable
    except ImportError as e:
        raise MinosImportException("Error importing Package")
